@@ -27,12 +27,15 @@
 #include <com/sun/star/awt/SystemDependentXWindow.hpp>
 #include <com/sun/star/awt/XSystemDependentWindowPeer.hpp>
 #include <com/sun/star/awt/XWindow2.hpp>
+#include <com/sun/star/awt/Key.hpp>
+#include <com/sun/star/awt/KeyModifier.hpp>
 #include <com/sun/star/beans/XPropertySet.hpp>
 #include <com/sun/star/container/XNameAccess.hpp>
 #include <com/sun/star/awt/XPopupMenu.hpp>
 #include <com/sun/star/awt/XMenuExtended.hpp>
 #include <com/sun/star/awt/XPopupMenuExtended.hpp>
 #include <com/sun/star/frame/XController.hpp>
+#include <com/sun/star/frame/XComponentLoader.hpp>
 #include <com/sun/star/frame/XDispatch.hpp>
 #include <com/sun/star/frame/XDispatchHelper.hpp>
 #include <com/sun/star/frame/XDispatchProvider.hpp>
@@ -56,6 +59,7 @@
 
 #include <gio/gio.h>
 #include <libdbusmenu-glib/client.h>
+#include <libdbusmenu-gtk/menuitem.h>
 
 using rtl::OUString;
 using rtl::OString;
@@ -75,6 +79,7 @@ using com::sun::star::beans::PropertyValue;
 using com::sun::star::container::XNameAccess;
 using com::sun::star::container::NoSuchElementException;
 using com::sun::star::frame::XController;
+using com::sun::star::frame::XComponentLoader;
 using com::sun::star::frame::XDispatch;
 using com::sun::star::frame::XDispatchProvider;
 using com::sun::star::frame::XDispatchHelper;
@@ -472,14 +477,19 @@ FrameHelper::getXID ()
 void
 FrameHelper::dispatchCommand (OUString command)
 {
-	Reference < XDispatchHelper > rDispatchHelper (xMSF->createInstance(OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.DispatchHelper"))),
-	                                               UNO_QUERY);
-	Reference < XDispatchHelper > xdh(rDispatchHelper);
+	OUString target = OUString::createFromAscii ("");
+	Reference < XDispatchHelper > xdh (xMSF->createInstance(OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.DispatchHelper"))),
+	                                   UNO_QUERY);
 	
+	if (command.matchAsciiL ("vnd.sun.star.popup:RecentFileList", 33, 0))
+	    target = OUString::createFromAscii ("_blank");		
+	
+	if (command.matchAsciiL ("private:factory/", 16, 0))
+	    target = OUString::createFromAscii ("_blank");
 	
 	xdh->executeDispatch (Reference < XDispatchProvider > (xFrame, UNO_QUERY),
 	                      command,
-	                      OUString::createFromAscii(""),
+	                      target,
 	                      0,
 	                      Sequence < PropertyValue > ());
 }
@@ -574,9 +584,10 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 		Reference < XMenuExtended > xMenuEx (xMenu, UNO_QUERY);
 		guint16 id = xMenu->getItemId (i);
 		OUString oUCommand = xMenuEx->getCommand (id);
+		OString command = OUStringToOString (oUCommand, RTL_TEXTENCODING_ASCII_US);
 		
 		DbusmenuMenuitem *item = DBUSMENU_MENUITEM(g_list_nth_data(items, i));
-		
+				
 		if (!item)
 			continue;
 
@@ -600,17 +611,36 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
        		continue;
 		}
 		
-		
-		// Replace the tilde with an underscore 
-		OString command = OUStringToOString (oUCommand, RTL_TEXTENCODING_ASCII_US);
+		//Setting the command
 		dbusmenu_menuitem_property_set (item, "CommandURL", command.getStr());
 
-/*		Sequence < KeyEvent > evs = findShortcutForCommand (oUCommand);
-		for (int j = 0; j < evs.getLength (); j++)
+		//Getting a shortcut
+		KeyEvent kev = findShortcutForCommand (oUCommand);
+		//g_debug ("%s: %x - %x - %x - %x", command.getStr(), kev.KeyCode, kev.KeyChar, kev.KeyFunc, kev.Modifiers);
+
+		if (kev.KeyCode != 0)  //KeyCode must have a value
 		{
-			//if KeyCode is 0 continue until we get a valid keycode
-			g_debug ("%s: %x - %x - %x - %x", command.getStr(), evs[i].KeyCode, evs[i].KeyChar, evs[i].KeyFunc, evs[i].Modifiers);
-		}*/
+			guint keycode;
+			GdkModifierType mod = (GdkModifierType)0;
+		
+			if (awt::KeyModifier::SHIFT & kev.Modifiers)
+				mod = (GdkModifierType)(mod | GDK_SHIFT_MASK);
+			if (awt::KeyModifier::MOD2 & kev.Modifiers)
+				mod = (GdkModifierType)(mod | GDK_MOD1_MASK); //Alt key
+			if (awt::KeyModifier::MOD1 & kev.Modifiers || awt::KeyModifier::MOD3 & kev.Modifiers)
+				mod = (GdkModifierType)(mod | GDK_CONTROL_MASK);
+
+			if (kev.KeyCode >= awt::Key::F1 && kev.KeyCode <= awt::Key::F1)
+			{
+				keycode = (guint)(kev.KeyCode - awt::Key::F1) + (guint)0xffbe;
+				dbusmenu_menuitem_property_set_shortcut (item, keycode, mod);
+			}
+			else if (kev.KeyCode >= awt::Key::A && kev.KeyCode <= awt::Key::Z)
+			{
+				keycode = (guint)(kev.KeyCode - awt::Key::A) + (guint)'a';
+				dbusmenu_menuitem_property_set_shortcut (item, keycode, mod);
+			}
+		}
 
 		MenuItemInfo* commInfo = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gconstpointer)command.getStr());
 		if (!commInfo)
@@ -807,34 +837,68 @@ FrameHelper::getAcceleratorConfigurations (Reference < XModel >        xModel,
 	this->globAccelConf = globAccelConf;
 }
 
-Sequence < KeyEvent >
+KeyEvent
 FrameHelper::findShortcutForCommand (OUString command)
 
 {
-	Sequence < KeyEvent > events (0);
+	KeyEvent kev;
+	OString com = OUStringToOString (command, RTL_TEXTENCODING_ASCII_US);
 
-	try
+	Sequence < OUString > commands (1);
+	commands[0] = command;
+
+	//TODO: The Help menu needs a special case for F1
+	//TODO: The Repeat command should always be taken from the global configuration
+	
+	try 
 	{
-		events = docAccelConf->getKeyEventsByCommand (command);
-	}
-	catch(...)
-	{
-		try
+		Sequence < Any > evs = docAccelConf->getPreferredKeyEventsForCommandList (commands);
+		
+
+		for (int j = 0; j < evs.getLength (); j++)
 		{
-			events = modAccelConf->getKeyEventsByCommand (command);
-		}
-		catch(...)
-		{
-			try 
+			KeyEvent ev;
+			if (evs[j] >>= ev)
 			{
-				events = globAccelConf->getKeyEventsByCommand (command);
-			}
-			catch(...)
-			{
-				return events;
+				//g_debug ("Document:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
+				return ev;
 			}
 		}
 	}
+	catch (...)
+	{}	
+	try 
+	{
+		Sequence < Any > evs = modAccelConf->getPreferredKeyEventsForCommandList (commands);
+		
 
-	return events;
+		for (int j = 0; j < evs.getLength (); j++)
+		{
+			KeyEvent ev;
+			if (evs[j] >>= ev)
+			{
+				//g_debug ("Module:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
+				return ev;
+			}
+		}
+	}
+	catch (...)
+	{}
+	try 
+	{
+		Sequence < Any > evs = globAccelConf->getPreferredKeyEventsForCommandList (commands);
+		
+		for (int j = 0; j < evs.getLength (); j++)
+		{
+			KeyEvent ev;
+			if (evs[j] >>= ev)
+			{
+				//g_debug ("Global:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
+				return ev;
+			}
+		}
+	}
+	catch (...)
+	{}
+	return kev;
 }
