@@ -269,10 +269,43 @@ FrameJob::exportMenus (Reference < XFrame > xFrame)
 	//Create dbusmenu server object path string
 	DbusmenuServer *server = dbusmenu_server_new (xid_to_object_path(getXID (xFrame)).getStr());
 
+
+	Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
+	Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
+	                                            UNO_QUERY);
+	if (!xLayoutManager.is())
+	{
+		g_object_unref (server);
+		return;
+	}
+
+	Reference < XUIElement > menuBar(xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
+	                                 UNO_QUERY);
+	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
+
+	Reference < XMenu > xMenu (menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")),
+	                           UNO_QUERY);
+	
+	if (!xMenu.is ())
+	{
+		g_object_unref (server);
+		return;
+	}
+
 	//Create a new frame helper	to close the server when needed
 	FrameHelper *helper = new FrameHelper (mxMSF, xFrame, server);
 	xFrame->addFrameActionListener (Reference < XFrameActionListener > (helper));
+	
+	//Getting the XMenu interface for the menubar
+	
 
+		
+
+	//Populate dbusmenu items and start the server
+	DbusmenuMenuitem *root = getRootMenuitem (xMenu, (gpointer)helper);
+	dbusmenu_server_set_root (server, root);
+	
+	
 	//Listen to the availability of the registrar
 	guint watcher = g_bus_watch_name (G_BUS_TYPE_SESSION,
 	                                  "com.canonical.AppMenu.Registrar",
@@ -281,23 +314,7 @@ FrameJob::exportMenus (Reference < XFrame > xFrame)
 	                                  on_registrar_unavailable,
 	                                  helper,
 	                                  NULL);
-
 	helper->setRegistrarWatcher (watcher);
-	
-	//Getting the XMenu interface for the menubar
-	Reference< XPropertySet > frameProps (xFrame, UNO_QUERY);
-	Reference < XLayoutManager > xLayoutManager(frameProps->getPropertyValue(OUString::createFromAscii("LayoutManager")),
-	                                            UNO_QUERY);
-	Reference < XUIElement > menuBar(xLayoutManager->getElement (OUString::createFromAscii("private:resource/menubar/menubar")),
-	                                 UNO_QUERY);
-	Reference < XPropertySet > menuPropSet (menuBar, UNO_QUERY);
-	
-	Reference < XMenu > xMenu;
-	menuPropSet->getPropertyValue(OUString::createFromAscii("XMenuBar")) >>= xMenu;
-
-	//Populate dbusmenu items and start the server
-	DbusmenuMenuitem *root = getRootMenuitem (xMenu, (gpointer)helper);
-	dbusmenu_server_set_root (server, root);
 }
 
 OUString FrameJob_getImplementationName ()
